@@ -96,6 +96,7 @@ struct SNP_Calling_Parameters{
 	unsigned int real_read_count;
 	unsigned int reported_SNPs;
 	unsigned int reported_indels;
+	int supporting_read_excessed_reported;
 };
 
 #define PRECALCULATE_FACTORIAL 2000000
@@ -212,17 +213,16 @@ int is_snp_bitmap(char * bm, unsigned int pos)
 	return bm [bytes] & (1 << bits);
 
 }
-void put_hash_to_pile(HashTable * merge_table, unsigned int* snp_voting_piles, struct SNP_Calling_Parameters * parameters)
+void put_hash_to_pile(HashTable * merge_table, unsigned int* snp_voting_piles, struct SNP_Calling_Parameters * parameters, char * chro, int block_start )
 {
 
 	int bucket;
 	KeyValuePair * cursor;
 
-	for(bucket=0; bucket< merge_table -> numOfBuckets; bucket++)
-	{
+	for(bucket=0; bucket< merge_table -> numOfBuckets; bucket++) {
 		cursor = merge_table -> bucketArray[bucket];
-		while (1)
-		{
+
+		while (1) {
 			if (!cursor) break;
 			int base_N_qual = cursor->value - NULL;
 			int POI_pos = cursor->key - NULL - 100;
@@ -232,20 +232,20 @@ void put_hash_to_pile(HashTable * merge_table, unsigned int* snp_voting_piles, s
 			int j;
 			unsigned int supporting_bases = 0;
 
-			for(j=0; j<4; j++)
-			{
+			for(j=0; j<4; j++) {
 				supporting_bases += snp_voting_piles[POI_pos*4+j];
 			}
 
-			if(supporting_bases < parameters->max_supporting_read_number)
-			{
-				if(qual1 <  (parameters -> is_phred_64?'@':'!')+parameters->min_phred_score) 	// low quality bases
-				{
+			if(supporting_bases < parameters->max_supporting_read_number) {
+				if(qual1 <  (parameters -> is_phred_64?'@':'!')+parameters->min_phred_score){	// low quality bases
 					// the low-seq-quality bases are not inclued in voting.
-				}
-				else
-				{
-					snp_voting_piles[POI_pos*4+base1]+=1;
+				} else snp_voting_piles[POI_pos*4+base1]+=1;
+			}else{
+				if(parameters -> supporting_read_excessed_reported < 100){
+					parameters -> supporting_read_excessed_reported++;
+					SUBREADprintf("Warning: the depth exceeded the limit of %d at %s : %d\n", parameters->max_supporting_read_number, chro ,  block_start + POI_pos);
+					if(parameters -> supporting_read_excessed_reported == 100)
+						SUBREADprintf("Too many warnings.\nNo more warning messages are reported.\n");
 				}
 			}
 
@@ -254,7 +254,7 @@ void put_hash_to_pile(HashTable * merge_table, unsigned int* snp_voting_piles, s
 	}
 }
 
-int read_tmp_block(struct SNP_Calling_Parameters * parameters, FILE * tmp_fp, char ** SNP_bitmap_recorder, unsigned int * snp_voting_piles, int block_no, unsigned int reference_len, char * referenced_genome)
+int read_tmp_block(struct SNP_Calling_Parameters * parameters, FILE * tmp_fp, char ** SNP_bitmap_recorder, unsigned int * snp_voting_piles, int block_no, unsigned int reference_len, char * referenced_genome, char * chro_name, unsigned int offset)
 {
 	int last_read_id=-1,i;
 	HashTable * merge_table = HashTableCreate(1000);
@@ -264,7 +264,7 @@ int read_tmp_block(struct SNP_Calling_Parameters * parameters, FILE * tmp_fp, ch
 
 	while(!feof(tmp_fp))
 	{
-		int type_char = fgetc(tmp_fp);
+		int type_char = fgetc(tmp_fp), rlen=-1;
 		if(type_char == EOF ) break;
 
 		fseek(tmp_fp, -1 , SEEK_CUR);
@@ -273,7 +273,11 @@ int read_tmp_block(struct SNP_Calling_Parameters * parameters, FILE * tmp_fp, ch
 		{
 			VCF_temp_read_t SNP_rec;
 
-			fread(&SNP_rec, sizeof(SNP_rec),1 , tmp_fp);
+			rlen = fread(&SNP_rec, sizeof(SNP_rec),1 , tmp_fp);
+			if(rlen < 1){
+				SUBREADputs("ERROR: the temporary file is broken.");
+				return -1;
+			}
 			if(!(*SNP_bitmap_recorder))
 			{
 				(*SNP_bitmap_recorder)=malloc((reference_len/8)+2);
@@ -292,10 +296,25 @@ int read_tmp_block(struct SNP_Calling_Parameters * parameters, FILE * tmp_fp, ch
 			char read[MAX_READ_LENGTH];
 			char qual[MAX_READ_LENGTH];
 
-			fread(&read_rec, sizeof(read_rec), 1, tmp_fp);
-			fread(&read_len, sizeof(short), 1, tmp_fp);
-			fread(read, sizeof(char), read_len, tmp_fp);
-			int rlen = fread(qual, sizeof(char), read_len, tmp_fp);
+			int rlen = fread(&read_rec, sizeof(read_rec), 1, tmp_fp);
+			if(rlen < 1){
+				SUBREADputs("ERROR: the temporary file is broken.");
+				return -1;
+			}
+
+			rlen = fread(&read_len, sizeof(short), 1, tmp_fp);
+			if(rlen < 1){
+				SUBREADputs("ERROR: the temporary file is broken.");
+				return -1;
+			}
+
+			rlen = fread(read, sizeof(char), read_len, tmp_fp);
+			if(rlen < read_len){
+				SUBREADputs("ERROR: the temporary file is broken.");
+				return -1;
+			}
+			
+			rlen = fread(qual, sizeof(char), read_len, tmp_fp);
 			if(rlen < read_len){
 				SUBREADputs("ERROR: the temporary file is broken.");
 				return -1;
@@ -314,7 +333,7 @@ int read_tmp_block(struct SNP_Calling_Parameters * parameters, FILE * tmp_fp, ch
 			if(parameters->is_paired_end_data){
 				if( (last_read_id >>1) != (read_rec.read_number>>1) && last_read_id>=0 && merge_table -> numOfElements > 0)
 				{
-					put_hash_to_pile(merge_table, snp_voting_piles, parameters);
+					put_hash_to_pile(merge_table, snp_voting_piles, parameters,chro_name, offset);
 					HashTableDestroy(merge_table);
 					merge_table = HashTableCreate(1000);
 				}
@@ -403,6 +422,13 @@ int read_tmp_block(struct SNP_Calling_Parameters * parameters, FILE * tmp_fp, ch
 						{
 							snp_voting_piles[(first_base_pos+i-1)*4+base_int]+=1;
 						}
+					}else{
+						if(parameters -> supporting_read_excessed_reported < 100){
+							parameters -> supporting_read_excessed_reported++;
+							SUBREADprintf("Warning: the depth exceeded the limit of %d at %s : %d\n", parameters->max_supporting_read_number, chro_name ,  offset + first_base_pos+i);
+							if(parameters -> supporting_read_excessed_reported == 100)
+								SUBREADprintf("Too many warnings.\nNo more warning messages are reported.\n");
+						}
 					}
 				}
 			}
@@ -411,7 +437,7 @@ int read_tmp_block(struct SNP_Calling_Parameters * parameters, FILE * tmp_fp, ch
 	}
 
 	if(parameters->is_paired_end_data && merge_table -> numOfElements > 0)
-		put_hash_to_pile(merge_table, snp_voting_piles, parameters);
+		put_hash_to_pile(merge_table, snp_voting_piles, parameters, chro_name, offset);
 	
 	HashTableDestroy(merge_table);
 
@@ -637,7 +663,7 @@ int process_snp_votes(FILE *out_fp, unsigned int offset , unsigned int reference
 		pcutoff_list[i]=-1.;
 	}
 
-	int read_is_error = read_tmp_block(parameters, tmp_fp,&SNP_bitmap_recorder,snp_voting_piles,block_no, reference_len, referenced_genome);
+	int read_is_error = read_tmp_block(parameters, tmp_fp,&SNP_bitmap_recorder,snp_voting_piles,block_no, reference_len, referenced_genome, chro_name, offset);
 
 	fclose(tmp_fp);
 	if (parameters -> delete_piles)
@@ -659,7 +685,7 @@ int process_snp_votes(FILE *out_fp, unsigned int offset , unsigned int reference
 		{
 			int min_phred_score_raw = parameters -> min_phred_score;
 			parameters -> min_phred_score -= 3;
-			read_tmp_block(parameters, tmp_fp,NULL , snp_BGC_piles,block_no, reference_len, referenced_genome);
+			read_tmp_block(parameters, tmp_fp,NULL , snp_BGC_piles,block_no, reference_len, referenced_genome, chro_name, offset);
 			parameters -> min_phred_score  = min_phred_score_raw;
 		}
 		fclose(tmp_fp);
@@ -1509,9 +1535,10 @@ void print_usage_snp(char * myname)
 	SUBREADputs("Optional arguments:");
 	SUBREADputs("");
 	SUBREADputs("  -a <file>  Provide a set of annotated SNPs (e.g. SNPs included in the dbSNP");
-	SUBREADputs("             database). The supplied file should be in VCF format. Providing");
-	SUBREADputs("             known SNPs to the program should improve its SNP calling");
-	SUBREADputs("             performance. Note that the provided SNPs may or may not be called.");
+	SUBREADputs("             database). The supplied file should be in VCF format (gzipped file");
+	SUBREADputs("             is accepted). Providing known SNPs to the program should improve");
+	SUBREADputs("             its SNP calling performance. Note that the provided SNPs may or");
+	SUBREADputs("             may not be called.");
 	SUBREADputs("");
 	SUBREADputs("  -b         Indicate the input file provided via -i is in BAM format.");
 	SUBREADputs("");
@@ -1529,10 +1556,9 @@ void print_usage_snp(char * myname)
 	SUBREADputs("  -r <int>   Specify the minimum number of mapped reads a SNP-containing");
 	SUBREADputs("             location must have (ie. the minimum coverage). 1 by default.");
 	SUBREADputs("");
-	SUBREADputs("  -x <int>   Specify the maximum number of mapped reads a SNP-containing");
-	SUBREADputs("             location have have. 1000 by default. Any location having more than");
-	SUBREADputs("             the threshold number of reads will not be considered for SNP");
-	SUBREADputs("             calling. This option is useful for removing PCR artefacts.");
+	SUBREADputs("  -x <int>   Specify the maximum depth a SNP location is allowed to have.");
+	SUBREADputs("             1,000,000 reads by default. This option is useful for removing PCR");
+	SUBREADputs("             artefacts.");
 	SUBREADputs("");
 	SUBREADputs("  -s <int>   Specify the minimum base quality scores (Phred scores) read bases");
 	SUBREADputs("             must satisfy to be used for SNP calling. 13 by default. Read bases");
@@ -1604,7 +1630,7 @@ int main_snp_calling_test(int argc,char ** argv)
 	parameters.supporting_read_rate = 0.;
 	parameters.min_supporting_read_number = 1;
 	parameters.min_alternative_read_number = 1;
-	parameters.max_supporting_read_number = 1000;
+	parameters.max_supporting_read_number = 1000000;
 	parameters.neighbour_filter_testlen = -1; 
 	parameters.neighbour_filter_rate = 0.000000001;
 	parameters.min_phred_score = 13;
@@ -1796,9 +1822,9 @@ int main_snp_calling_test(int argc,char ** argv)
 
 	print_in_box(80,1,1,"exactSNP setting");
 	print_in_box(80,0,1,"");
-	print_in_box(80,0,0,"                  Input file : %s (%s)", in_SAM_file, parameters.is_BAM_file_input?"BAM":"SAM");
-	print_in_box(80,0,0,"                 Output file : %s", out_BED_file);
-	print_in_box(80,0,0,"            Reference genome : %s", in_FASTA_file);
+	print_in_box(80,0,0,"                  Input file : %s (%s)", get_short_fname(in_SAM_file), parameters.is_BAM_file_input?"BAM":"SAM");
+	print_in_box(80,0,0,"                 Output file : %s", get_short_fname(out_BED_file));
+	print_in_box(80,0,0,"            Reference genome : %s", get_short_fname(in_FASTA_file));
 	print_in_box(80,0,1,"");
 	print_in_box(80,0,0,"                     Threads : %d", threads);
 	print_in_box(80,0,0,"        Min supporting reads : %d", parameters.min_supporting_read_number);
@@ -1810,7 +1836,7 @@ int main_snp_calling_test(int argc,char ** argv)
 	print_in_box(80,0,0,"         P value upper bound : %.5f", parameters.cutoff_upper_bound);
 	print_in_box(80,0,0,"       Flanking windows size : %d", parameters.fisher_exact_testlen);
 	if(parameters.known_SNP_vcf[0])
-		print_in_box(80,0,0,"       Known SNP annotations : %s", parameters.known_SNP_vcf);
+		print_in_box(80,0,0,"       Known SNP annotations : %s", get_short_fname(parameters.known_SNP_vcf));
 	
 	print_in_box(80,0,1,"");
 	print_in_box(80,2,1,"http://subread.sourceforge.net/");
