@@ -145,7 +145,7 @@ int transfer_SAM_to_assembly(global_context_t * global_context)
 		int read_len, is_repeated, mapping_quality, is_anchor_certain=1;
 		int pos_delta = 0;
 
-		if(feof(global_context -> input_reads.first_read_file.input_fp))break;
+		if(feof((FILE *)global_context -> input_reads.first_read_file.input_fp))break;
 
 		read_text[0]=0;
 		qual_text[0]=0;
@@ -326,7 +326,7 @@ int main(int argc, char ** argv)
 
 	struct timeval xtime; 
 	gettimeofday(&xtime,NULL);
-	srand(time(NULL)^xtime.tv_usec);
+	myrand_srand(time(NULL)^xtime.tv_usec);
 
 
 
