@@ -48,6 +48,7 @@ static struct option long_options[] =
 	{"SAMinput", no_argument, 0, 0},
 	{"reportPairedMultiBest",  no_argument, 0, 0},
 	{"sv", no_argument, 0, 0},
+	{"longDel", no_argument, 0, 0},
 	{"extraColumns",  no_argument, 0, 0},
 	{"forcedPE",  no_argument, 0, 0},
 	{"ignoreUnmapped",  no_argument, 0, 0},
@@ -60,6 +61,8 @@ static struct option long_options[] =
 	{"minVoteCutoff",  required_argument, 0, 0},
 	{"maxRealignLocations",  required_argument, 0, 0},
 	{"multiMapping", no_argument, 0, 0},
+	{"keepReadOrder", no_argument, 0, 0},
+	{"sortReadsByCoordinates", no_argument, 0, 0},
 	{0, 0, 0, 0}
 };
 
@@ -168,6 +171,17 @@ void print_usage_core_aligner()
 	SUBREADputs("");
 	SUBREADputs("  --rg <string>     Add <tag:value> to the read group (RG) header in the output.");
 	SUBREADputs("");
+	SUBREADputs("# read order");
+	SUBREADputs("");
+	SUBREADputs("  --keepReadOrder   Keep order of reads in BAM output the same as that in the");
+	SUBREADputs("                    input file. Reads from the same pair are always placed next");
+	SUBREADputs("                    to each other no matter this option is specified or not.");
+	SUBREADputs("");
+	SUBREADputs("  --sortReadsByCoordinates Output location-sorted reads. This option is");
+	SUBREADputs("                    applicable for BAM output only. A BAI index file is also");
+	SUBREADputs("                    generated for each BAM file so the BAM files can be directly");
+	SUBREADputs("                    loaded into a genome browser.");
+	SUBREADputs("");
 	SUBREADputs("# color space reads");
 	SUBREADputs("");
 	SUBREADputs("  -b                Convert color-space read bases to base-space read bases in");
@@ -196,8 +210,9 @@ void print_usage_core_aligner()
 	SUBREADputs("");
 	SUBREADputs("# gene annotation");
 	SUBREADputs("");
-	SUBREADputs("  -a                Name of an annotation file. GTF/GFF format by default. See");
-	SUBREADputs("                    -F option for more format information.");
+	SUBREADputs("  -a                Name of an annotation file (gzipped file is accepted).");
+	SUBREADputs("                    GTF/GFF format by default. See -F option for more format");
+	SUBREADputs("                    information.");
 	SUBREADputs("");
 	SUBREADputs("  -F                Specify format of the provided annotation file. Acceptable");
 	SUBREADputs("                    formats include 'GTF' (or compatible GFF format) and");
@@ -239,6 +254,8 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 	opterr = 1;
 	optopt = 63;
 
+	subread_rebuild_cmd(argc, argv, global_context);
+
 	global_context->config.entry_program_name = CORE_PROGRAM_SUBREAD;
 	global_context->config.max_mismatch_exonic_reads = 3;
 	global_context->config.max_mismatch_junction_reads = 3;
@@ -465,6 +482,10 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 				{
 					global_context->config.report_multi_mapping_reads = 1;
 				}
+				else if(strcmp("ignoreUnmapped", long_options[option_index].name)==0) 
+				{
+					global_context->config.ignore_unmapped_reads = 1;
+				}
 				else if(strcmp("memoryMultiplex", long_options[option_index].name)==0) 
 				{
 					global_context->config.memory_use_multiplex = atof(optarg);
@@ -484,15 +505,33 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 				}
 				else if(strcmp("BAMinput", long_options[option_index].name)==0) 
 				{
+					if(global_context->config.is_SAM_file_input && global_context->config.is_BAM_input == 0){
+						SUBREADprintf("ERROR: you cannot specify both SAMinput and BAMinput\n");
+						return -1;
+					}
+
 					global_context->config.is_BAM_input = 1;
 					global_context->config.is_SAM_file_input = 1;
 				}
+				else if(strcmp("keepReadOrder", long_options[option_index].name)==0) 
+				{
+					global_context->config.is_input_read_order_required=1;
+				}
+				else if(strcmp("sortReadsByCoordinates", long_options[option_index].name)==0) 
+				{
+					global_context->config.sort_reads_by_coordinates=1;
+				}
 				else if(strcmp("extraColumns", long_options[option_index].name)==0) 
 				{
 					global_context->config.SAM_extra_columns=1;
 				}
 				else if(strcmp("SAMinput", long_options[option_index].name)==0) 
 				{
+					if(global_context->config.is_SAM_file_input && global_context->config.is_BAM_input == 1){
+						SUBREADprintf("ERROR: you cannot specify both SAMinput and BAMinput\n");
+						return -1;
+					}
+
 					global_context->config.is_BAM_input = 0;
 					global_context->config.is_SAM_file_input = 1;
 				}
@@ -504,10 +543,6 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 				{
 					global_context -> config.fast_run = 1;
 				}
-				else if(strcmp("ignoreUnmapped", long_options[option_index].name)==0) 
-				{
-					global_context->config.ignore_unmapped_reads = 1;
-				}
 				else if(strcmp("sv", long_options[option_index].name)==0) 
 				{
 					global_context->config.do_breakpoint_detection = 1;
@@ -515,6 +550,12 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 					global_context->config.prefer_donor_receptor_junctions = 0;
 					//global_context->config.do_big_margin_filtering_for_reads = 1;
 				}
+				else if(strcmp("longDel", long_options[option_index].name)==0) 
+				{
+					global_context->config.do_breakpoint_detection = 1;
+					global_context->config.do_long_del_detection = 1;
+					global_context->config.prefer_donor_receptor_junctions = 0;
+				}
 				else if(strcmp("minMappedLength", long_options[option_index].name)==0) 
 				{
 					if(!is_valid_digit(optarg, "minMappedLength"))
@@ -553,7 +594,7 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 				{
 					global_context->config.maximise_sensitivity_indel = 1;
 					global_context->config.realignment_minimum_variant_distance = 1;
-					global_context->config.max_indel_length = 16;
+				//	global_context->config.max_indel_length = max(16, global_context->config.max_indel_length);
 				}
 				else if(strcmp("minVoteCutoff", long_options[option_index].name)==0)
 				{
@@ -577,7 +618,7 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 	if(global_context->config.reported_multi_best_reads > 1 && ! global_context->config.report_multi_mapping_reads)
 		SUBREADprintf("WARNING: You required multi best alignments, but disallowed multi-mapping reads. You need to turn on the multi-mapping option.\n");
 
-	global_context->config.more_accurate_fusions = global_context->config.more_accurate_fusions && global_context->config.do_fusion_detection;
+	global_context->config.more_accurate_fusions = global_context->config.more_accurate_fusions && (global_context->config.do_fusion_detection || global_context->config.do_long_del_detection);
 	if(global_context->config.more_accurate_fusions)
 	{
 		global_context->config.high_quality_base_threshold = 999999;
