@@ -67,6 +67,8 @@ static struct option long_options[] =
 	{"minMappedFraction",  required_argument, 0, 0},
 	{"complexIndels", no_argument, 0, 0},
 	{"multiMapping", no_argument, 0, 0},
+	{"keepReadOrder", no_argument, 0, 0},
+	{"sortReadsByCoordinates", no_argument, 0, 0},
 	{0, 0, 0, 0}
 };
 
@@ -171,6 +173,17 @@ void print_usage_core_subjunc()
 	SUBREADputs("");
 	SUBREADputs("  --rg <string>     Add <tag:value> to the read group (RG) header in the output.");
 	SUBREADputs("");
+	SUBREADputs("# read order");
+	SUBREADputs("");
+	SUBREADputs("  --keepReadOrder   Keep order of reads in BAM output the same as that in the");
+	SUBREADputs("                    input file. Reads from the same pair are always placed next");
+	SUBREADputs("                    to each other no matter this option is specified or not.");
+	SUBREADputs("");
+	SUBREADputs("  --sortReadsByCoordinates Output location-sorted reads. This option is");
+	SUBREADputs("                    applicable for BAM output only. A BAI index file is also");
+	SUBREADputs("                    generated for each BAM file so the BAM files can be directly");
+	SUBREADputs("                    loaded into a genome browser.");
+	SUBREADputs("");
 	SUBREADputs("# color space reads");
 	SUBREADputs("");
 	SUBREADputs("  -b                Convert color-space read bases to base-space read bases in");
@@ -199,8 +212,9 @@ void print_usage_core_subjunc()
 	SUBREADputs("");
 	SUBREADputs("# gene annotation");
 	SUBREADputs("");
-	SUBREADputs("  -a                Name of an annotation file. GTF/GFF format by default. See");
-	SUBREADputs("                    -F option for more format information.");
+	SUBREADputs("  -a                Name of an annotation file (gzipped file is accepted).");
+	SUBREADputs("                    GTF/GFF format by default. See -F option for more format");
+	SUBREADputs("                    information.");
 	SUBREADputs("");
 	SUBREADputs("  -F                Specify format of the provided annotation file. Acceptable");
 	SUBREADputs("                    formats include 'GTF' (or compatible GFF format) and");
@@ -241,6 +255,8 @@ int parse_opts_subjunc(int argc , char ** argv, global_context_t * global_contex
 	opterr = 1;
 	optopt = 63;
 
+	subread_rebuild_cmd(argc, argv, global_context);
+
 	global_context->config.entry_program_name = CORE_PROGRAM_SUBJUNC;
 	global_context->config.max_mismatch_exonic_reads = 3;
 	global_context->config.max_mismatch_junction_reads = 3;
@@ -498,11 +514,19 @@ int parse_opts_subjunc(int argc , char ** argv, global_context_t * global_contex
 				}
 				else if(strcmp("BAMinput", long_options[option_index].name)==0) 
 				{
+					if(global_context->config.is_SAM_file_input && global_context->config.is_BAM_input == 0){
+						SUBREADprintf("Error: you cannot specify both SAMinput and BAMinput\n");
+						return -1;
+					}
 					global_context->config.is_BAM_input = 1;
 					global_context->config.is_SAM_file_input = 1;
 				}
 				else if(strcmp("SAMinput", long_options[option_index].name)==0) 
 				{
+					if(global_context->config.is_SAM_file_input && global_context->config.is_BAM_input == 1){
+						SUBREADprintf("Error: you cannot specify both SAMinput and BAMinput\n");
+						return -1;
+					}
 					global_context->config.is_BAM_input = 0;
 					global_context->config.is_SAM_file_input = 1;
 				}
@@ -514,6 +538,14 @@ int parse_opts_subjunc(int argc , char ** argv, global_context_t * global_contex
 					global_context->config.limited_tree_scan = 0;
 					global_context->config.max_insertion_at_junctions = atoi(optarg);
 				}
+				else if(strcmp("keepReadOrder", long_options[option_index].name)==0) 
+				{
+					global_context->config.is_input_read_order_required=1;
+				}
+				else if(strcmp("sortReadsByCoordinates", long_options[option_index].name)==0) 
+				{
+					global_context->config.sort_reads_by_coordinates=1;
+				}
 				else if(strcmp("extraColumns", long_options[option_index].name)==0) 
 				{
 					global_context->config.SAM_extra_columns=1;
@@ -583,7 +615,7 @@ int parse_opts_subjunc(int argc , char ** argv, global_context_t * global_contex
 				{
 					global_context->config.maximise_sensitivity_indel = 1;
 					global_context->config.realignment_minimum_variant_distance = 1;
-					global_context->config.max_indel_length = 16;
+				//	global_context->config.max_indel_length = 16;
 				}
 				else if(strcmp("disableBigMargin", long_options[option_index].name)==0)
 				{
