@@ -172,6 +172,8 @@ typedef struct{
 	int downscale_mapping_quality;
 	int is_BAM_input;
 	int is_BAM_output;
+	int is_input_read_order_required;
+	int sort_reads_by_coordinates;
 	int convert_color_to_base;
 	int SAM_extra_columns;
 	int report_multiple_best_in_pairs;
@@ -215,6 +217,7 @@ typedef struct{
 
 	// subfusion
 	int do_fusion_detection;
+	int do_long_del_detection;
 	int do_structural_variance_detection;
 	int prefer_donor_receptor_junctions;
 	int more_accurate_fusions;
@@ -455,12 +458,7 @@ typedef struct{
 	void * module_thread_contexts[5];
 	gene_value_index_t * current_value_index;
 
-	output_fragment_buffer_t * output_buffer;
-	int output_buffer_item;
-	int output_buffer_pointer;
-	int is_finished;
 	subread_lock_t output_lock;
-
 	unsigned int all_mapped_reads;
 	unsigned int not_properly_pairs_wrong_arrangement;
 	unsigned int not_properly_pairs_different_chro;
@@ -500,11 +498,12 @@ typedef struct{
 	// running contexts
 	void * module_contexts[5]; 
 	thread_context_t * all_thread_contexts;
-	int last_written_fragment_number;
+	subread_read_number_t last_written_fragment_number;
 	int need_merge_buffer_now;
 	read_input_t input_reads;
 	bigtable_t bigtable;
-
+	int rebuilt_command_line_size;
+	char * rebuilt_command_line;
 
 	subread_lock_t bigtable_lock;
 	subread_lock_t output_lock;
@@ -670,4 +669,5 @@ int is_valid_float(char * optarg, char * optname);
 int exec_cmd(char * cmd, char * outstr, int out_limit);
 int is_pos_in_annotated_exon_regions(global_context_t * global_context, unsigned int pos);
 char * get_sam_chro_name_from_alias(HashTable * tab, char * anno_chro);
+void subread_rebuild_cmd(int argc, char ** argv, global_context_t * global_context);
 #endif
