@@ -18,6 +18,8 @@ typedef unsigned int coverage_bin_entry_t;
 int is_BAM_input = 0;
 int max_M = 10;
 int paired_end = 0;
+int ignore_bad_pair = 0;
+int base_values = 0;
 char input_file_name[300];
 char output_file_name[300];
 HashTable * cov_bin_table;
@@ -100,7 +102,7 @@ void add_chro(char *sam_h)
 	}
 
 	void ** bin_entry = malloc(sizeof(void *)*2);
-	coverage_bin_entry_t * chro_bin = calloc(sizeof(coverage_bin_entry_t) , chro_len);
+	coverage_bin_entry_t * chro_bin = calloc(sizeof(coverage_bin_entry_t) , chro_len * ( base_values?5:1 ));
 	if(!chro_bin)
 	{
 		SUBREADprintf("ERROR: cannot allocate the memory block. You need at least 4GB of memory,\n");
@@ -111,7 +113,7 @@ void add_chro(char *sam_h)
 	SUBREADprintf("Added a new chromosome : %s [%u]\n", chro_name, chro_len);
 }
 
-void get_read_info(char * fl, char * chro, unsigned int * pos , char * cigar, int *flags){
+void get_read_info(char * fl, char * chro, unsigned int * pos_0base , char * cigar, int *flags, int * tlen, char ** rtext){
 	char * tmp_tok = NULL;
 
 	char * tmp_res = strtok_r(fl, "\t", &tmp_tok);
@@ -122,12 +124,28 @@ void get_read_info(char * fl, char * chro, unsigned int * pos , char * cigar, in
 	tmp_res = strtok_r(NULL, "\t", &tmp_tok); // chro
 	strcpy(chro, tmp_res);
 
-	tmp_res = strtok_r(NULL, "\t", &tmp_tok); // pos
-	(*pos) = atoi(tmp_res);
+	tmp_res = strtok_r(NULL, "\t", &tmp_tok); // pos_0base
+	(*pos_0base) = atoi(tmp_res);
 
 	tmp_res = strtok_r(NULL, "\t", &tmp_tok); // qual
 	tmp_res = strtok_r(NULL, "\t", &tmp_tok); // cigar
 	strcpy(cigar, tmp_res);
+
+	tmp_res = strtok_r(NULL, "\t", &tmp_tok); // mate chro
+	tmp_res = strtok_r(NULL, "\t", &tmp_tok); // mate pos
+	tmp_res = strtok_r(NULL, "\t", &tmp_tok); // TLEN 
+	(*tlen) = atoi(tmp_res);
+
+	if(rtext)
+		*rtext = strtok_r(NULL, "\t", &tmp_tok);
+}
+
+int base_value_to_int(char b){
+	if(b=='A') return 1;
+	if(b=='C') return 2;
+	if(b=='G') return 3;
+	if(b=='T') return 4;
+	return 0;
 }
 
 int covCalc()
@@ -138,9 +156,9 @@ int covCalc()
 	HashTableSetKeyComparisonFunction(cov_bin_table , fc_strcmp_chro);
 	HashTableSetDeallocationFunctions(cov_bin_table , free, free);
 
-	unsigned int hit_locs[2][MAX_FRAGMENT_LENGTH], reads = 0;
+	unsigned int this_hit_locs[MAX_FRAGMENT_LENGTH], reads = 0;
+	char this_hit_bases[MAX_FRAGMENT_LENGTH];
 	coverage_bin_entry_t * chrbin12[2];
-	int hits1,hits2;
 	SamBam_FILE * in_fp = SamBam_fopen(input_file_name, is_BAM_input?SAMBAM_FILE_BAM:SAMBAM_FILE_SAM);
 	char * line_buffer = malloc(3000);
 	
@@ -151,25 +169,29 @@ int covCalc()
 		if(line_buffer[0]=='@'){
 			if(strstr(line_buffer,"@SQ\t"))
 				add_chro(line_buffer);
-		}
-		else
-		{
+		} else {
 			char * Chros[ max_M ];
 			unsigned int Staring_Points[max_M];
 			unsigned short Staring_Read_Points[max_M];
 			unsigned short Section_Lengths[max_M];
 	
-			int flags=0, x1, is_junc = 0;
+			int flags=0, x1, is_junc = 0, tlen = 0;
 			char cigar_str[200];
-			char chro[200];
-			unsigned int pos = 0;
+			char chro[200], *rtext = NULL;
+			unsigned int pos_0base = 0;
 			cigar_str[0]=0;
 			chro[0]=0;
+			int this_hits = 0;
 
-			if(reads % 2 == 0) hits1= hits2 = 0;
-			get_read_info(line_buffer, chro, &pos, cigar_str, &flags);
+			get_read_info(line_buffer, chro, &pos_0base, cigar_str, &flags, &tlen, &rtext);
+			pos_0base --;
 
-			if(flags & 4) continue;
+			if(0 && FIXLENstrcmp("NS500643:166:HNCCHBGXY:1:23105:1714:15599" , line_buffer)==0)
+				SUBREADprintf("INPUT READ_%d: FLAG=%d, chro=%s, pos=%u\n", 1+(reads % 2 ), flags, chro, pos_0base);
+			if((flags & 4) || ( paired_end && (tlen == 0 || abs(tlen) > 500000) )){
+				reads++;
+				continue;
+			}
 
 			void ** bin_entry = HashTableGet(cov_bin_table, chro);
 			if(NULL == bin_entry)
@@ -179,10 +201,8 @@ int covCalc()
 
 			coverage_bin_entry_t * chrbin = (coverage_bin_entry_t*) bin_entry[0];
 			unsigned int chrlen = (void *)( bin_entry[1]) - NULL;
-			int cigar_sections = RSubread_parse_CIGAR_string(chro, pos, cigar_str, max_M, Chros, Staring_Points, Staring_Read_Points, Section_Lengths, &is_junc);
+			int cigar_sections = RSubread_parse_CIGAR_string(chro, pos_0base, cigar_str, max_M, Chros, Staring_Points, Staring_Read_Points, Section_Lengths, &is_junc);
 			if(paired_end) {
-				int * this_hits = (reads%2)?&hits2:&hits1;
-				unsigned int * this_hit_locs = &(hit_locs[reads%2][0]);
 				for(x1 = 0; x1 < cigar_sections; x1++){
 					unsigned int x2,x3;
 
@@ -190,49 +210,66 @@ int covCalc()
 					//	SUBREADprintf("Cigar [%d] = %u ~ + %u ; this_hits=%d\n", x1, Staring_Points[x1], Section_Lengths[x1], *this_hits);
 					//}
 
+					int read_cursor = Staring_Read_Points[x1];
 					for(x2 = Staring_Points[x1]; x2<Staring_Points[x1]+Section_Lengths[x1]; x2++){
 						int found = 0;
-						for(x3=0; x3<*this_hits; x3++){
+						for(x3=0; x3<this_hits; x3++){
 							if(this_hit_locs[x3] == x2){
 								found =1;
 								break;
 							}
 						}
-						if(!found) this_hit_locs[(*this_hits)++] = x2;
-						if( *this_hits >= MAX_FRAGMENT_LENGTH){
+						if(!found){
+							this_hit_bases[this_hits] = base_value_to_int(rtext[read_cursor]);
+							this_hit_locs[this_hits++] = x2;
+						}
+						if(this_hits >= MAX_FRAGMENT_LENGTH){
 							SUBREADprintf("ERROR: read is too long : %s!\n", cigar_str);
 							return -1;
 						}
+						read_cursor ++;
 					}
 				}
 
-				if(* this_hits > 0) assert(chrbin);
+				if(this_hits > 0) assert(chrbin);
 				chrbin12[reads%2] = chrbin;
 			} else for(x1 = 0; x1 < cigar_sections; x1++) {
 				unsigned int x2;
-				for(x2 = Staring_Points[x1]; x2<Staring_Points[x1]+Section_Lengths[x1]; x2++)
-				{
+				if(0 && FIXLENstrcmp("NS500643:166:HNCCHBGXY:4:23507:11772:18798" , line_buffer)==0)
+					SUBREADprintf("TEST_ONE %s [%d] : %u ~ %u  (chr_len=%u)  ; read_seq = [%d]  %s\n", line_buffer, x1, Staring_Points[x1], x2<Staring_Points[x1]+Section_Lengths[x1], chrlen,  Staring_Read_Points[x1], rtext +  Staring_Read_Points[x1]);
+
+				if(0 && strcmp("chr12",chro) == 0 && Staring_Points[x1] <= 113994516 && Staring_Points[x1] + Section_Lengths[x1] > 113994516){
+					int rpos_0 = 113994516 - Staring_Points[x1] + Staring_Read_Points[x1];
+					SUBREADprintf("    TEST_BASE : %s : val=%c\n", line_buffer, rtext[rpos_0]);
+				}
+
+				int read_cursor = Staring_Read_Points[x1];
+				for(x2 = Staring_Points[x1]; x2<Staring_Points[x1]+Section_Lengths[x1]; x2++) {
 					if(x2 < chrlen) {
-						if(chrbin[x2] <= COVERAGE_MAX_INT)chrbin[x2] ++;
+						if(base_values){
+							chrbin[x2*5 + base_value_to_int(rtext[read_cursor])] ++;
+							if(0 && FIXLENstrcmp("NS500643:166:HNCCHBGXY:4:23507:11772:18798" , line_buffer)==0)
+								SUBREADprintf("    TEST_MY_BASE : [%d 0_base] = %c (%d) ; res=%d\n", x2, rtext[read_cursor],  base_value_to_int(rtext[read_cursor]), chrbin[x2*5 + base_value_to_int(rtext[read_cursor])]);
+						}
+						else if(chrbin[x2] <= COVERAGE_MAX_INT)chrbin[x2] ++;
 						all_counted ++;
 						if(all_counted % 10000000 == 0)
 							SUBREADprintf("Processed %llu bases.\n", all_counted);
 					} else {
-						SUBREADprintf("%s:%s %u [%s] :: %u-%u\n", line_buffer , chro, pos, cigar_str, Staring_Points[x1], Staring_Points[x1]+Section_Lengths[x1]);
+						SUBREADprintf("%s:%s %u [%s] :: %u-%u\n", line_buffer , chro, pos_0base, cigar_str, Staring_Points[x1], Staring_Points[x1]+Section_Lengths[x1]);
 						SUBREADprintf("Read %s overhangs the boundary of chromosome %s (%u >= %u)\n", line_buffer, chro, x2, chrlen);
 						//exit(-1);
 					}
+					read_cursor ++;
 				}
 			}
 			if(reads % 2 == 1){
 				int r,x1;
 				for(r = 0; r < 2; r++){
-					int hits = r?hits2:hits1;
-					unsigned int * this_hit_locs = &hit_locs[r][0];
-					if(hits>0)assert(chrbin12[r]);
-					for(x1 = 0; x1<hits;x1++){
-					//	SUBREADprintf("pos[%d : %d,%d]=%u %p\n",x1,hits1,hits2, this_hit_locs[x1], this_hit_locs);
-						chrbin12[r][this_hit_locs[x1]]++;
+					if(this_hits>0)assert(chrbin12[r]);
+					for(x1 = 0; x1<this_hits;x1++){
+						if(base_values) chrbin12[r][this_hit_locs[x1]*5 + this_hit_bases[x1]]++;
+						else chrbin12[r][this_hit_locs[x1]]++;
 					}
 				}
 			}
@@ -262,7 +299,11 @@ int covCalc()
 			sprintf(out_name,"%s-%s.bin", output_file_name, chro);
 
 			FILE * fpo = fopen(out_name,"w");
-			fwrite(chrbin, sizeof(coverage_bin_entry_t), chrlen, fpo);
+			//SUBREADprintf("PTR %s = %p\n", chro, chrbin);
+			if(0 && strcmp(chro, "chr10")==0){
+				SUBREADprintf("WRITE chr10 : %u ptr %p\n", chrbin [75631235-1], chrbin);
+			}
+			fwrite(chrbin, sizeof(coverage_bin_entry_t), chrlen * (base_values?5:1), fpo);
 			fclose(fpo);
 			free(chrbin);
 
@@ -298,14 +339,20 @@ int cov_calc_main(int argc, char ** argv)
 	opterr=1;
 	optopt=63;
 
-	while ((c = getopt_long (argc, argv, "BpM:i:o:?", cov_calc_long_options, &option_index)) != -1)
+	while ((c = getopt_long (argc, argv, "bCBpM:i:o:?", cov_calc_long_options, &option_index)) != -1)
 		switch(c)
 		{
+			case 'C':
+				ignore_bad_pair = 1;
+				break;
 			case 'M':
 				if(!is_valid_digit_range(optarg, "maxMOp", 1 , 64))
 					exit(-1);
 				max_M = atoi(optarg);
 			break;
+			case 'b':
+				base_values = 1;
+			break;
 			case 'p':
 				paired_end = 1;
 			break;
