@@ -28,6 +28,7 @@
 #include <sys/types.h>
 #include "hashtable.h"
 #include "gene-value-index.h"
+#include "HelperFunctions.h"
 #include "gene-algorithms.h"
 #include "sorted-hashtable.h"
 #include "input-files.h"
@@ -471,7 +472,7 @@ int add_repeated_subread(gehash_t * tab , unsigned int subr, unsigned char ** hu
 
 int scan_gene_index(const char index_prefix [], char ** chro_files, int chro_file_number, int threshold, gehash_t *huge_table)
 {
-	int file_number, table_no, i ,j;
+	int file_number, i ,j;
 	int status = NEXT_FILE;
 	unsigned int offset, read_no;
 	double local_begin_ftime = miltime();
@@ -520,7 +521,6 @@ int scan_gene_index(const char index_prefix [], char ** chro_files, int chro_fil
 
 	file_number = 0;
 	offset = 0;
-	table_no = 0;
 	read_no = 0;
 
 	char * fn = malloc(3100);
@@ -1047,7 +1047,6 @@ int main_buildindex(int argc,char ** argv)
 				IS_COLOR_SPACE = 1;
 				break;
 			case 'M':
-				IS_FORCED_ONE_BLOCK = 0;
 				memory_limit = atoi(optarg);
 				break;
 			case 'f':
@@ -1141,7 +1140,7 @@ int main_buildindex(int argc,char ** argv)
 	SUBREADputs("");
 	print_in_box(80, 1, 1, "indexBuilder setting");
 	print_in_box(80, 0, 1, "");
-	print_in_box(80, 0, 0, "               Index name : %s", output_file);
+	print_in_box(80, 0, 0, "               Index name : %s", get_short_fname(output_file));
 	print_in_box(80, 0, 0, "              Index space : %s", IS_COLOR_SPACE?"color-space":"base-space");
 
 	if(IS_FORCED_ONE_BLOCK)
@@ -1172,7 +1171,7 @@ int main_buildindex(int argc,char ** argv)
 		if(f_type != FILE_TYPE_FASTA){
 			o_char = '?';
 		}
-		print_in_box(94, 0, 0, "                            %c[32m%c%c[36m %s%c[0m", CHAR_ESC, o_char, CHAR_ESC,  fasta_fn , CHAR_ESC);
+		print_in_box(94, 0, 0, "                            %c[32m%c%c[36m %s%c[0m", CHAR_ESC, o_char, CHAR_ESC,  get_short_fname(fasta_fn) , CHAR_ESC);
 	}
 	print_in_box(80, 0, 0, "");
 	print_in_box(80, 2, 1, "http://subread.sourceforge.net/");
@@ -1207,7 +1206,11 @@ int main_buildindex(int argc,char ** argv)
 	if(tmp_fa_file[0]==0)strcpy(tmp_fa_file, "./");
 
 	sprintf(tmp_fa_file+strlen(tmp_fa_file), "/subread-index-sam-%06u-XXXXXX", getpid());
-	mkstemp(tmp_fa_file);
+	int tmpfdd = mkstemp(tmp_fa_file);
+	if(tmpfdd == -1){
+		SUBREADprintf("ERROR: cannot create temp file\n");
+		return -1;
+	}
 
 	sprintf(log_file_name, "%s.log", output_file);
 	FILE * log_fp = f_subr_open(log_file_name,"w");
