@@ -37,13 +37,14 @@
 
 int gehash_create(gehash_t * the_table, size_t expected_size, char is_small_table)
 {
-	return gehash_create_ex(the_table, expected_size, is_small_table, SUBINDEX_VER0, 3);
+	return gehash_create_ex(the_table, expected_size, is_small_table, SUBINDEX_VER0, 3, 0);
 }
-int gehash_create_ex(gehash_t * the_table, size_t expected_size, char is_small_table, int version_number, int index_gap)
+int gehash_create_ex(gehash_t * the_table, size_t expected_size, char is_small_table, int version_number, int index_gap, int padding)
 {
 	int expected_bucket_number;
 	int i;
 
+	memset(the_table, 0, sizeof(gehash_t));
 	if(expected_size ==0)
 		expected_size = GEHASH_DEFAULT_SIZE;
 
@@ -86,6 +87,7 @@ int gehash_create_ex(gehash_t * the_table, size_t expected_size, char is_small_t
 			  expected_bucket_number *
   			  sizeof(struct gehash_bucket )
 			);
+	the_table -> padding = padding;
 
 	if(!the_table -> buckets)
 	{
@@ -176,8 +178,8 @@ int _gehash_resize_bucket(gehash_t * the_table , int bucket_no, char is_small_ta
 		current_bucket->space_size = new_bucket_length;
 	}
 	return 0;
-
 }
+
 int _gehash_resize_bucket_old(gehash_t * the_table , int bucket_no, char is_small_table)
 {
 	int new_bucket_length;
@@ -188,11 +190,12 @@ int _gehash_resize_bucket_old(gehash_t * the_table , int bucket_no, char is_smal
 
 	if (current_bucket->space_size<1)
 	{
+		int randv = myrand_rand();
 		if(is_small_table)
 			//new_bucket_length = (int)max(15,current_bucket->space_size*1.5+1);
-			new_bucket_length = (int)(GEHASH_BUCKET_LENGTH*(1.5+3.*pow((rand()*1./RAND_MAX),30)));
+			new_bucket_length = (int)(GEHASH_BUCKET_LENGTH*(1.5+3.*pow(randv * 1./RAND_MAX,30)));
 		else
-			new_bucket_length = (int)(GEHASH_BUCKET_LENGTH*(1.+3.*pow((rand()*1./RAND_MAX),30)));
+			new_bucket_length = (int)(GEHASH_BUCKET_LENGTH*(1. +3.*pow(randv * 1./RAND_MAX,30)));
 
 
 	//	printf("NS1=%d\nNS2=%d\n", new_bucket_length, new_bucket_length);
@@ -231,7 +234,7 @@ int _gehash_resize_bucket_old(gehash_t * the_table , int bucket_no, char is_smal
 	}
 	else
 	{
-		int test_start = rand() % the_table-> buckets_number;
+		int test_start = myrand_rand() % the_table-> buckets_number;
 		int i;
 		int need_allowc = 1;
 		for (i =test_start; i<test_start+10000; i++)
@@ -239,7 +242,7 @@ int _gehash_resize_bucket_old(gehash_t * the_table , int bucket_no, char is_smal
 			if (i==bucket_no)continue;
 			if (i>=the_table-> buckets_number)
 			{
-				test_start = rand() % the_table-> buckets_number;
+				test_start = myrand_rand() % the_table-> buckets_number;
 				i = test_start;
 				continue;
 			} 
@@ -440,9 +443,9 @@ int gehash_insert_limited(gehash_t * the_table, gehash_key_t key, gehash_data_t
 	if(occurance >= limit) 
 	{
 
-		if(rand()%32768 < replace_prob)
+		if(myrand_rand()%32768 < replace_prob)
 			return 1;
-		int replacement_id = rand()%occurance;
+		int replacement_id = myrand_rand()%occurance;
 		occurance = 0;
 		for(xk1=0; xk1<current_bucket->current_items ; xk1++)
 			if(current_bucket->item_keys[xk1]==key)
@@ -474,6 +477,7 @@ int gehash_insert_limited(gehash_t * the_table, gehash_key_t key, gehash_data_t
 
 size_t gehash_go_q_tolerable(gehash_t * the_table, gehash_key_t key, int offset, int read_len, int is_reversed, gene_vote_t * vote,gene_vote_number_t weight, gene_quality_score_t quality ,int max_match_number, int indel_tolerance, int subread_number, int max_error_bases, int subread_len, unsigned int low_border, unsigned int high_border)
 {
+	assert(max_error_bases < 5);
 	char error_pos_stack[10];	// max error bases = 10;
 
 	gehash_key_t mutation_key;
@@ -539,6 +543,7 @@ size_t gehash_go_q_tolerable(gehash_t * the_table, gehash_key_t key, int offset,
 
 size_t gehash_go_q_CtoT(gehash_t * the_table, gehash_key_t key, int offset, int read_len, int is_reversed, gene_vote_t * vote, gene_vote_number_t weight, gene_quality_score_t quality ,int max_match_number, int indel_tolerance, int subread_number, int max_error_bases, unsigned int low_border, unsigned int high_border)
 {
+	assert(max_error_bases < 5);
 	int error_pos_stack[10];	// max error bases = 10;
 	int C_poses[16];
 	int  availableC=0;
@@ -561,6 +566,7 @@ size_t gehash_go_q_CtoT(gehash_t * the_table, gehash_key_t key, int offset, int
 
 	for (error_bases= min(availableC,max_error_bases); error_bases <= min(availableC,max_error_bases); error_bases++)
 	{
+		assert(error_bases < 5);
 		int j;
 		for(i=0; i<error_bases; i++)
 		{
@@ -569,7 +575,6 @@ size_t gehash_go_q_CtoT(gehash_t * the_table, gehash_key_t key, int offset, int
 
 		while (1)
 		{
-
 			char mutation_stack[10];
 			memset(mutation_stack, 0 , error_bases);
 			while(1)
@@ -633,6 +638,7 @@ void assign_best_vote(gene_vote_t * vote, int i, int j)
 
 size_t gehash_go_q(gehash_t * the_table, gehash_key_t raw_key, int offset, int read_len, int is_reversed, gene_vote_t * vote, gene_quality_score_t quality, int indel_tolerance, int subread_number, unsigned int low_border, unsigned int high_border)
 {
+	//SUBREADprintf("Q=%u, OFFSET=%d, B=%u ~ %u\n", raw_key, offset, low_border, high_border);
 
 	if(the_table->version_number == SUBINDEX_VER0)
 	{
@@ -704,7 +710,6 @@ size_t gehash_go_q(gehash_t * the_table, gehash_key_t raw_key, int offset, int r
 						gene_vote_number_t test_max = (vote->votes[offsetX][i]);
 						test_max += 1;
 						vote->votes[offsetX][i] = test_max;
-						vote->quality[offsetX][i] += quality;
 						if (offset_from_5 <  vote->coverage_start [offsetX][i])
 							vote->coverage_start [offsetX][i] = offset_from_5;
 						if (offset_from_5 +16 > vote->coverage_end [offsetX][i])
@@ -727,7 +732,6 @@ size_t gehash_go_q(gehash_t * the_table, gehash_key_t raw_key, int offset, int r
 					vote -> items[offsetX] ++;
 					dat[i] = kv;
 					vote->votes[offsetX][i]=1;
-					vote->quality[offsetX][i]=quality;
 					vote->masks[offsetX][i]= (is_reversed?IS_NEGATIVE_STRAND:0);
 					vote->coverage_start [offsetX][i] = offset_from_5;
 					vote->coverage_end [offsetX][i] = offset_from_5+16;
@@ -759,56 +763,68 @@ size_t gehash_go_q(gehash_t * the_table, gehash_key_t raw_key, int offset, int r
 
 					for (i=0;i<datalen;i++)
 					{
-
-						if((!WITHOUT_CLUSTER_ORDERING ) && subread_number + 1 <= vote -> last_subread_cluster[offsetX][i])
-							continue;
-
 						int di = dat[i];
 						int dist0 = kv-di;
-						if( dist0 >= -indel_tolerance && dist0 <= indel_tolerance )
-						{
-							if(is_reversed  == (0!=(vote -> masks[offsetX][i]&IS_NEGATIVE_STRAND)))
-							{
-								gene_vote_number_t test_max = (vote->votes[offsetX][i]);
-								test_max += 1;
-								vote -> votes[offsetX][i] = test_max;
-								vote -> quality[offsetX][i] += quality;
-
-
-								/*
-								if (offset_from_5 <  vote->coverage_start [offsetX][i])
-								{
-									vote->coverage_start [offsetX][i] = offset_from_5;
-								}*/
-								if (offset_from_5 +16 > vote->coverage_end [offsetX][i])
-								{
-									vote->coverage_end [offsetX][i] = offset_from_5+16;
-								}
-
-								int toli =  vote -> toli[offsetX][i];
-
-								if (dist0 !=  vote->current_indel_cursor[offsetX][i])
-								{
-									toli +=3;
-									if (toli < indel_tolerance*3)
-									{
-										vote -> toli[offsetX][i] = toli;
-										vote -> indel_recorder[offsetX][i][toli] = subread_number+1; 
-										vote -> indel_recorder[offsetX][i][toli+1] = subread_number+1;
-										vote -> indel_recorder[offsetX][i][toli+2] = dist0; 
-											
-										if(toli < indel_tolerance*3-3) vote -> indel_recorder[offsetX][i][toli+3]=0;
-									}
-									vote->current_indel_cursor [offsetX][i] = (char)dist0;
-								}
-								else
-									vote -> indel_recorder[offsetX][i][toli+1] = subread_number+1;
-
-								vote -> last_subread_cluster[offsetX][i] = subread_number + 1;
-
-								vote->max_vote = max(vote->max_vote , test_max);
-								i = 9999999;
+						if( dist0 >= -indel_tolerance && dist0 <= indel_tolerance && is_reversed  == (0!=(vote -> masks[offsetX][i]&IS_NEGATIVE_STRAND))) {
+							int toli =  vote -> toli[offsetX][i];
+	
+							if(di >= 46494104 && di <= 46496104 ){
+								SUBREADprintf("VOTES: at %u, subread_no = %d , last_cluster = %d , toli = %d\n", di, subread_number , vote -> last_subread_cluster[offsetX][i] , toli);
 							}
+
+							if( toli > 0 && subread_number + 1 == vote -> last_subread_cluster[offsetX][i] ){
+								int move_dist = 0;
+								if( toli >=3 ) move_dist = vote -> indel_recorder[offsetX][i][toli-3+2];
+								int new_dist = move_dist;
+								move_dist -= vote -> indel_recorder[offsetX][i][toli+2];
+								new_dist -= dist0;
+								if(abs(move_dist) > abs(new_dist)){
+									toli -= 3;
+									vote -> toli[offsetX][i] = toli;
+									vote -> last_subread_cluster[offsetX][i]--;
+									vote -> votes[offsetX][i] --;
+								}
+							}
+							if((!WITHOUT_CLUSTER_ORDERING ) && subread_number + 1 <= vote -> last_subread_cluster[offsetX][i])
+								continue;
+
+
+							gene_vote_number_t test_max = (vote->votes[offsetX][i]);
+							test_max += 1;
+							vote -> votes[offsetX][i] = test_max;
+
+							/*
+							if (offset_from_5 <  vote->coverage_start [offsetX][i])
+							{
+								vote->coverage_start [offsetX][i] = offset_from_5;
+							}*/
+							if (offset_from_5 +16 > vote->coverage_end [offsetX][i])
+							{
+								vote->coverage_end [offsetX][i] = offset_from_5+16;
+							}
+
+
+							if (dist0 !=  vote->current_indel_cursor[offsetX][i])
+							{
+								toli +=3;
+								if (toli < indel_tolerance*3)
+								{
+									vote -> toli[offsetX][i] = toli;
+									vote -> indel_recorder[offsetX][i][toli] = subread_number+1; 
+									vote -> indel_recorder[offsetX][i][toli+1] = subread_number+1;
+									vote -> indel_recorder[offsetX][i][toli+2] = dist0; 
+										
+									if(toli < indel_tolerance*3-3) vote -> indel_recorder[offsetX][i][toli+3]=0;
+								}
+								vote->current_indel_cursor [offsetX][i] = (char)dist0;
+							}
+							else
+								vote -> indel_recorder[offsetX][i][toli+1] = subread_number+1;
+
+							vote -> last_subread_cluster[offsetX][i] = subread_number + 1;
+
+							vote->max_vote = max(vote->max_vote , test_max);
+							i = 9999999;
 						}
 					}
 					if (i>=9999999){
@@ -830,7 +846,6 @@ size_t gehash_go_q(gehash_t * the_table, gehash_key_t raw_key, int offset, int r
 						vote -> items[offsetX2] ++;
 						dat2[datalen2] = kv;
 						vote -> masks[offsetX2][datalen2]=(is_reversed?IS_NEGATIVE_STRAND:0);
-						vote -> quality[offsetX2][datalen2]=quality;
 						vote -> votes[offsetX2][datalen2]=1;
 						vote -> toli[offsetX2][datalen2]=0;
 						vote -> last_subread_cluster[offsetX2][datalen2] = subread_number + 1;
@@ -874,7 +889,8 @@ size_t gehash_go_q(gehash_t * the_table, gehash_key_t raw_key, int offset, int r
 		
 		if(!items) return 0;
 
-		int imin=0, imax=items;
+//#warning "======== MAKE SURE THAT '-1' IS CORRECT ============"
+		int imin=0, imax=items - 1;
 		int last_accepted_index;
 
 		while(1)
@@ -938,24 +954,44 @@ size_t gehash_go_q(gehash_t * the_table, gehash_key_t raw_key, int offset, int r
 
 					for (i=0;i<datalen;i++)
 					{
-						if((!WITHOUT_CLUSTER_ORDERING ) &&  subread_number + 1 <= vote -> last_subread_cluster[offsetX][i]) continue;
 						int di = dat[i];
 						int dist0 = kv-di;
-						if( dist0 >= -indel_tolerance && dist0 <= indel_tolerance )
+
+	
+						if( dist0 >= -indel_tolerance && dist0 <= indel_tolerance && is_reversed  == (0!=(vote -> masks[offsetX][i]&IS_NEGATIVE_STRAND)) )
 						{
+
+//							if(di >= 46494104 && di <= 46496104 ){
+//								SUBREADprintf("VOTES: at %u, subread_no = %d , last_cluster = %d , toli = %d\n", di, subread_number , vote -> last_subread_cluster[offsetX][i] , toli);
+//							}
+
+							int toli =  vote -> toli[offsetX][i];
+							if( toli > 0 && subread_number + 1 == vote -> last_subread_cluster[offsetX][i] ){
+								int move_dist = 0;
+								if( toli >=3 ) move_dist = vote -> indel_recorder[offsetX][i][toli-3+2];
+								int new_dist = move_dist;
+								move_dist -= vote -> indel_recorder[offsetX][i][toli+2];
+								new_dist -= dist0;
+								if(abs(move_dist) > abs(new_dist)){
+									toli -= 3;
+									vote -> toli[offsetX][i] = toli;
+									vote -> last_subread_cluster[offsetX][i]--;
+									vote -> votes[offsetX][i] --;
+								}
+							}
+
+							if((!WITHOUT_CLUSTER_ORDERING ) &&  subread_number + 1 <= vote -> last_subread_cluster[offsetX][i]) continue;
+
 							//SUBREADprintf("IIX = %d, BASE=%u, change_dist=%d, subread=#%d\n", iix, dat[i], dist0, subread_number);
-							if(is_reversed  == (0!=(vote -> masks[offsetX][i]&IS_NEGATIVE_STRAND)))
-							{
+							if(1){
 
 								gene_vote_number_t test_max = (vote->votes[offsetX][i]);
 								test_max += 1;
 								vote -> votes[offsetX][i] = test_max;
-								vote -> quality[offsetX][i] += quality;
 
 								if (offset +16 > vote->coverage_end [offsetX][i])
 									vote->coverage_end [offsetX][i] = offset+16;
 
-								int toli =  vote -> toli[offsetX][i];
 
 								if (dist0 !=  vote->current_indel_cursor[offsetX][i])
 								{
@@ -996,7 +1032,6 @@ size_t gehash_go_q(gehash_t * the_table, gehash_key_t raw_key, int offset, int r
 						vote -> items[offsetX2] ++;
 						dat2[datalen2] = kv;
 						vote -> masks[offsetX2][datalen2]=(is_reversed?IS_NEGATIVE_STRAND:0);
-						vote -> quality[offsetX2][datalen2]=quality;
 						vote -> votes[offsetX2][datalen2]=1;
 						vote -> toli[offsetX2][datalen2]=0;
 
@@ -1032,7 +1067,6 @@ void select_best_vote(gene_vote_t * vote)
 			if(vote -> votes[i][j] == vote->max_vote)
 			{
 				vote -> max_position = vote -> pos[i][j];
-				vote -> max_quality = vote -> quality[i][j];
 				vote -> max_mask = vote -> masks[i][j];
 				vote -> max_coverage_start = vote->coverage_start[i][j];
 				vote -> max_coverage_end = vote->coverage_end[i][j];
@@ -1233,7 +1267,7 @@ unsigned int load_int32(FILE * fp)
 	int ret;
 	int read_length;
 	read_length = fread(&ret, sizeof(int), 1, fp);
-	assert(read_length>0);
+	if(read_length<=0)assert(0);
 	return ret;
 }
 
@@ -1242,16 +1276,53 @@ long long int load_int64(FILE * fp)
 	long long int ret;
 	int read_length;
 	read_length = fread(&ret, sizeof(long long int), 1, fp);
-	assert(read_length>0);
+	if(read_length<=0)assert(0);
 	return ret;
 }
 
 
+int gehash_load_option(const char fname [], int option_no, int * result){
+	char tabname[MAX_FILE_NAME_LENGTH];
+	char magic_chars[8];
+	int found = 0, rrtv;
+	sprintf(tabname, "%s.00.b.tab", fname);
+	FILE * fp = f_subr_open(tabname, "rb");
+	if(fp == NULL){
+		sprintf(tabname, "%s.00.c.tab", fname);
+		fp = f_subr_open(tabname, "rb");
+	}
+	if(fp){	
+		rrtv = fread(magic_chars,1,8,fp);
+		if(rrtv < 8) return -1;
+		if(memcmp(magic_chars, "2subindx",7)==0) {
+			while(1) {
+				short option_key, option_length;
 
+				rrtv = fread(&option_key, 2, 1, fp);
+				if(rrtv < 1) return -1;
+				if(!option_key) break;
+
+				rrtv = fread(&option_length, 2, 1, fp);
+				if(rrtv < 1) return -1;
+
+				if(option_key == option_no){
+					*result = 0;
+					rrtv = fread(result ,option_length,1,fp);
+					if(rrtv < 1) return -1;
+					found = 1;
+				}
+				else
+					fseek(fp, option_length, SEEK_CUR);
+			}
+		}
+		fclose(fp);
+	}
+	return found;
+}
 
 int gehash_load(gehash_t * the_table, const char fname [])
 {
-	int i, read_length;
+	int i, read_length, rrtv;
 	char magic_chars[8];
 	magic_chars[7]=0;
 
@@ -1260,11 +1331,12 @@ int gehash_load(gehash_t * the_table, const char fname [])
 	FILE * fp = f_subr_open(fname, "rb");
 	if (!fp)
 	{
-		SUBREADprintf ("Table file `%s' is not found.\n", fname);
+		SUBREADprintf ("Table file '%s' is not found.\n", fname);
 		return 1;
 	}
 
-	fread(magic_chars,1,8,fp);
+	rrtv = fread(magic_chars,1,8,fp);
+	if(rrtv < 8) return -1;
 
 	if(memcmp(magic_chars, "2subindx",8)!=0)
 	{
@@ -1287,15 +1359,20 @@ int gehash_load(gehash_t * the_table, const char fname [])
 			{
 				short option_key, option_length;
 
-				fread(&option_key, 2, 1, fp);
+				rrtv = fread(&option_key, 2, 1, fp);
+				if(rrtv < 1) return -1;
 				if(!option_key) break;
 
-				fread(&option_length, 2, 1, fp);
-
+				rrtv = fread(&option_length, 2, 1, fp);
+				if(rrtv < 1) return -1;
+				rrtv = 999;
 				if(option_key == SUBREAD_INDEX_OPTION_INDEX_GAP)
-					fread(&(the_table -> index_gap),2,1,fp);
+					rrtv = fread(&(the_table -> index_gap),2,1,fp);
+				else if (option_key ==SUBREAD_INDEX_OPTION_INDEX_PADDING)
+					rrtv = fread(&(the_table -> padding),2,1,fp);
 				else
 					fseek(fp, option_length, SEEK_CUR);
+				if(rrtv <1) return -1;
 			}
 			assert(the_table -> index_gap);
 		}
@@ -1303,6 +1380,10 @@ int gehash_load(gehash_t * the_table, const char fname [])
 			the_table -> index_gap = 3;
 
 		the_table -> current_items = load_int64(fp);
+		if(the_table -> current_items < 1 || the_table -> current_items > 0xffffffffllu){
+			SUBREADputs("ERROR: the index format is unrecognizable.");
+			return 1;
+		}
 		the_table -> buckets_number = load_int32(fp);
 		the_table -> buckets = (struct gehash_bucket * )malloc(sizeof(struct gehash_bucket) * the_table -> buckets_number);
 		if(!the_table -> buckets)
@@ -1330,9 +1411,15 @@ int gehash_load(gehash_t * the_table, const char fname [])
 			if(current_bucket -> current_items > 0)
 			{
 				read_length = fread(current_bucket -> new_item_keys, sizeof(short), current_bucket -> current_items, fp);
-				assert(read_length>0);
+				if(read_length < current_bucket -> current_items){
+					SUBREADprintf("ERROR: the index is incomplete : %d < %u.\n",read_length, current_bucket -> current_items);
+					return 1;
+				}
 				read_length = fread(current_bucket -> item_values, sizeof(gehash_data_t), current_bucket -> current_items, fp);
-				assert(read_length>0);
+				if(read_length < current_bucket -> current_items){
+					SUBREADprintf("ERROR: the index value is incomplete : %d < %u.\n",read_length, current_bucket -> current_items);
+					return 1;
+				}
 			}
 
 		}
@@ -1377,9 +1464,15 @@ int gehash_load(gehash_t * the_table, const char fname [])
 			if(current_bucket -> current_items > 0)
 			{
 				read_length = fread(current_bucket -> item_keys, sizeof(gehash_key_t), current_bucket -> current_items, fp);
-				assert(read_length>0);
+				if(read_length < current_bucket -> current_items){
+					SUBREADprintf("ERROR: the index is incomplete.\n");
+					return 1;
+				}
 				read_length = fread(current_bucket -> item_values, sizeof(gehash_data_t), current_bucket -> current_items, fp);
-				assert(read_length>0);
+				if(read_length < current_bucket -> current_items){
+					SUBREADprintf("ERROR: the index is incomplete.\n");
+					return 1;
+				}
 			}
 
 		}
@@ -1437,6 +1530,12 @@ void write_options(FILE * fp, gehash_t * the_table)
 	short option_key, option_length;
 	short option_value;
 
+	option_key = SUBREAD_INDEX_OPTION_INDEX_PADDING;
+	option_length = 2;
+	option_value = the_table -> padding;
+
+	write_cell(option_key, option_length, (char *) &option_value,fp);
+
 	option_key = SUBREAD_INDEX_OPTION_INDEX_GAP;
 	option_length = 2;
 	option_value = the_table -> index_gap;
@@ -1455,7 +1554,7 @@ int gehash_dump(gehash_t * the_table, const char fname [])
 	int maximum_bucket_size = 0;
 	if (!fp)
 	{
-		SUBREADprintf ("Table file `%s' is not able to open.\n", fname);
+		SUBREADprintf ("Table file '%s' is not able to open.\n", fname);
 		return -1;
 	}
 
@@ -1604,13 +1703,25 @@ int gehash_dump(gehash_t * the_table, const char fname [])
 			}
 		}
 
-		fwrite(& (current_bucket -> current_items), sizeof(int), 1, fp);
-		fwrite(& (current_bucket -> space_size), sizeof(int), 1, fp);
+		int is_full = 0;
+		int write_len = fwrite(& (current_bucket -> current_items), sizeof(int), 1, fp);
+		if(write_len<1) is_full = 1;
+		write_len = fwrite(& (current_bucket -> space_size), sizeof(int), 1, fp);
+		if(write_len<1) is_full = 1;
+
 		if(the_table->version_number == SUBINDEX_VER0)
 			fwrite(current_bucket -> item_keys, sizeof(gehash_key_t), current_bucket -> current_items, fp);
-		else
-			fwrite(current_bucket -> new_item_keys, sizeof(short), current_bucket -> current_items, fp);
-		fwrite(current_bucket -> item_values, sizeof(gehash_data_t), current_bucket -> current_items, fp);
+		else{
+			write_len = fwrite(current_bucket -> new_item_keys, sizeof(short), current_bucket -> current_items, fp);
+			if(write_len < current_bucket -> current_items) is_full = 1;
+		}
+		write_len = fwrite(current_bucket -> item_values, sizeof(gehash_data_t), current_bucket -> current_items, fp);
+		if(write_len < current_bucket -> current_items) is_full = 1;
+		if(is_full){
+			fclose(fp);
+			SUBREADprintf("ERROR: Unable to write into the output file. Please check the disk space in the output directory.\n");
+			return 1;
+		}
 	}
 
 	if(the_table->version_number > SUBINDEX_VER0)
@@ -1623,8 +1734,13 @@ int gehash_dump(gehash_t * the_table, const char fname [])
 	}
 
 
-	fwrite(&(the_table -> is_small_table), sizeof(char), 1, fp);
+	int write_len = fwrite(&(the_table -> is_small_table), sizeof(char), 1, fp);
 	fclose(fp);
+
+	if(write_len < 1){
+		SUBREADprintf("ERROR: Unable to write into the output file. Please check the disk space in the output directory.\n");
+		return 1;
+	}
 	print_in_box(80,0,0,"");
 	return 0;
 }
