@@ -44,6 +44,9 @@
 #define SAM_FLAG_DUPLICATE 0x400
 #define SAM_FLAG_UNMAPPED 0x04
 
+#define SUBREAD_MAX_ULONGLONG 0xffffffffffffffffllu
+#define SUBREAD_MAX_LONGLONG 0x7fffffffffffffffll
+
 #define FUSION_BREAK_POINT	2
 #define FUSION_JUNCTION		1
 #define SPLICING_JUNCTION	0
@@ -486,6 +489,8 @@ struct explorer_record_t
 };
 
 FILE * f_subr_open(const char * fname, const char * mode);
+void myrand_srand(unsigned long long seed);
+int myrand_rand();
 #define abs(a) 	  ((a)>=0?(a):-(a))
 #define max(a,b)  ((a)<(b)?(b):(a))
 #define min(a,b)  ((a)>(b)?(b):(a))
